@@ -1,7 +1,9 @@
+// Copyright Contributors to the Open Cluster Management project
+
 package apis
 
 import (
-	v1 "github.com/stolostron/multiclusterhub-operator/pkg/apis/operator/v1"
+	v1 "github.com/open-cluster-management/multiclusterhub-operator/pkg/apis/operator/v1"
 )
 
 func init() {
