@@ -1,54 +1,60 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package multiclusterhub
 
 import (
 	"context"
 	"encoding/json"
-	e "errors"
 	"fmt"
+	"os"
 	"reflect"
+	"strings"
+
+	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 
 	"time"
 
 	subrelv1 "github.com/open-cluster-management/multicloud-operators-subscription-release/pkg/apis/apps/v1"
-	operatorsv1 "github.com/stolostron/multiclusterhub-operator/pkg/apis/operator/v1"
-	"github.com/stolostron/multiclusterhub-operator/pkg/foundation"
-	"github.com/stolostron/multiclusterhub-operator/pkg/helmrepo"
-	"github.com/stolostron/multiclusterhub-operator/pkg/manifest"
-	"github.com/stolostron/multiclusterhub-operator/pkg/subscription"
-	"github.com/stolostron/multiclusterhub-operator/pkg/utils"
-	"github.com/stolostron/multiclusterhub-operator/version"
+	operatorsv1 "github.com/open-cluster-management/multiclusterhub-operator/pkg/apis/operator/v1"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/channel"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/foundation"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/helmrepo"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/manifest"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/subscription"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/utils"
+	"github.com/open-cluster-management/multiclusterhub-operator/version"
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
-	v1 "k8s.io/api/core/v1"
 
+	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
+	"k8s.io/apimachinery/pkg/api/equality"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
-	"k8s.io/client-go/discovery"
 	"sigs.k8s.io/controller-runtime/pkg/client"
-	"sigs.k8s.io/controller-runtime/pkg/client/config"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 )
 
 // CacheSpec ...
 type CacheSpec struct {
-	IngressDomain     string
-	ImageOverrides    map[string]string
-	ImageOverrideType manifest.OverrideType
-	ImageRepository   string
-	ImageSuffix       string
-	ManifestVersion   string
-	ImageOverridesCM  string
+	IngressDomain    string
+	ImageOverrides   map[string]string
+	ImageRepository  string
+	ManifestVersion  string
+	ImageOverridesCM string
 }
 
 func (r *ReconcileMultiClusterHub) ensureDeployment(m *operatorsv1.MultiClusterHub, dep *appsv1.Deployment) (*reconcile.Result, error) {
 	dplog := log.WithValues("Deployment.Namespace", dep.Namespace, "Deployment.Name", dep.Name)
 
+	if utils.ProxyEnvVarsAreSet() {
+		dep = addProxyEnvVarsToDeployment(dep)
+	}
+
 	// See if deployment already exists and create if it doesn't
 	found := &appsv1.Deployment{}
 	err := r.client.Get(context.TODO(), types.NamespacedName{
@@ -134,6 +140,80 @@ func (r *ReconcileMultiClusterHub) ensureService(m *operatorsv1.MultiClusterHub,
 		return &reconcile.Result{}, err
 	}
 
+	modified := resourcemerge.BoolPtr(false)
+	existingCopy := found.DeepCopy()
+	resourcemerge.EnsureObjectMeta(modified, &existingCopy.ObjectMeta, s.ObjectMeta)
+	selectorSame := equality.Semantic.DeepEqual(existingCopy.Spec.Selector, s.Spec.Selector)
+
+	typeSame := false
+	requiredIsEmpty := len(s.Spec.Type) == 0
+	existingCopyIsCluster := existingCopy.Spec.Type == corev1.ServiceTypeClusterIP
+	if (requiredIsEmpty && existingCopyIsCluster) || equality.Semantic.DeepEqual(existingCopy.Spec.Type, s.Spec.Type) {
+		typeSame = true
+	}
+
+	if selectorSame && typeSame && !*modified {
+		return nil, nil
+	}
+
+	existingCopy.Spec.Selector = s.Spec.Selector
+	existingCopy.Spec.Type = s.Spec.Type
+	err = r.client.Update(context.TODO(), existingCopy)
+	if err != nil {
+		svlog.Error(err, "Failed to update Service")
+		return &reconcile.Result{}, err
+	}
+	return nil, nil
+}
+
+func (r *ReconcileMultiClusterHub) ensureAPIService(m *operatorsv1.MultiClusterHub, s *apiregistrationv1.APIService) (*reconcile.Result, error) {
+	svlog := log.WithValues("Service.Name", s.Name)
+
+	found := &apiregistrationv1.APIService{}
+	err := r.client.Get(context.TODO(), types.NamespacedName{
+		Name: s.Name,
+	}, found)
+	if err != nil && errors.IsNotFound(err) {
+
+		// Create the apiService
+		err = r.client.Create(context.TODO(), s)
+
+		if err != nil {
+			// Creation failed
+			svlog.Error(err, "Failed to create new apiService")
+			return &reconcile.Result{}, err
+		}
+
+		// Creation was successful
+		svlog.Info("Created a new apiService")
+		condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionTrue, NewComponentReason, "Created new resource")
+		SetHubCondition(&m.Status, *condition)
+		return nil, nil
+
+	} else if err != nil {
+		// Error that isn't due to the apiService not existing
+		svlog.Error(err, "Failed to get apiService")
+		return &reconcile.Result{}, err
+	}
+
+	modified := resourcemerge.BoolPtr(false)
+	existingCopy := found.DeepCopy()
+
+	resourcemerge.EnsureObjectMeta(modified, &existingCopy.ObjectMeta, s.ObjectMeta)
+	serviceSame := equality.Semantic.DeepEqual(existingCopy.Spec.Service, s.Spec.Service)
+	prioritySame := existingCopy.Spec.VersionPriority == s.Spec.VersionPriority && existingCopy.Spec.GroupPriorityMinimum == s.Spec.GroupPriorityMinimum
+	insecureSame := existingCopy.Spec.InsecureSkipTLSVerify == s.Spec.InsecureSkipTLSVerify
+
+	if !*modified && serviceSame && prioritySame && insecureSame {
+		return nil, nil
+	}
+
+	existingCopy.Spec = s.Spec
+	err = r.client.Update(context.TODO(), existingCopy)
+	if err != nil {
+		svlog.Error(err, "Failed to update apiService")
+		return &reconcile.Result{}, err
+	}
 	return nil, nil
 }
 
@@ -151,7 +231,6 @@ func (r *ReconcileMultiClusterHub) ensureChannel(m *operatorsv1.MultiClusterHub,
 		Namespace: m.Namespace,
 	}, found)
 	if err != nil && errors.IsNotFound(err) {
-
 		// Create the Channel
 		err = r.client.Create(context.TODO(), u)
 		if err != nil {
@@ -172,12 +251,28 @@ func (r *ReconcileMultiClusterHub) ensureChannel(m *operatorsv1.MultiClusterHub,
 		return &reconcile.Result{}, err
 	}
 
+	updated, needsUpdate := channel.Validate(m, found)
+	if needsUpdate {
+		selog.Info("Updating channel")
+		err = r.client.Update(context.TODO(), updated)
+		if err != nil {
+			// Update failed
+			selog.Error(err, "Failed to update channel")
+			return &reconcile.Result{}, err
+		}
+		return nil, nil
+	}
+
 	return nil, nil
 }
 
 func (r *ReconcileMultiClusterHub) ensureSubscription(m *operatorsv1.MultiClusterHub, u *unstructured.Unstructured) (*reconcile.Result, error) {
 	obLog := log.WithValues("Namespace", u.GetNamespace(), "Name", u.GetName(), "Kind", u.GetKind())
 
+	if utils.ProxyEnvVarsAreSet() {
+		u = addProxyEnvVarsToSub(u)
+	}
+
 	found := &unstructured.Unstructured{}
 	found.SetGroupVersionKind(schema.GroupVersionKind{
 		Group:   "apps.open-cluster-management.io",
@@ -286,77 +381,6 @@ func (r *ReconcileMultiClusterHub) ensureUnstructuredResource(m *operatorsv1.Mul
 	return nil, nil
 }
 
-func (r *ReconcileMultiClusterHub) apiReady(gv schema.GroupVersion) (*reconcile.Result, error) {
-	cfg, err := config.GetConfig()
-	if err != nil {
-		log.Error(err, "Failed to create rest config")
-		return &reconcile.Result{}, err
-	}
-
-	c, err := discovery.NewDiscoveryClientForConfig(cfg)
-	if err != nil {
-		log.Error(err, "Failed to create discovery client")
-		return &reconcile.Result{}, err
-	}
-
-	err = discovery.ServerSupportsVersion(c, gv)
-	if err != nil {
-		// Wait a little and try again
-		log.Info("Waiting for API group to be available", "API group", gv)
-		// condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionTrue, NewComponentReason, "Waiting for cert manager CRD availability")
-		// SetHubCondition(&m.Status, *condition)
-		return &reconcile.Result{RequeueAfter: time.Second * 10}, nil
-	}
-	return nil, nil
-}
-
-func (r *ReconcileMultiClusterHub) copyPullSecret(m *operatorsv1.MultiClusterHub, newNS string) (*reconcile.Result, error) {
-	sublog := log.WithValues("Copying Secret to cert-manager namespace", m.Spec.ImagePullSecret, "Namespace.Name", utils.CertManagerNamespace)
-
-	if m.Spec.ImagePullSecret == "" {
-		err := e.New("imagePullSecret is empty")
-		sublog.Error(err, "copyPullSecret requires a valid secret to copy")
-		return &reconcile.Result{}, err
-	}
-
-	pullSecret := &v1.Secret{}
-	err := r.client.Get(context.TODO(), types.NamespacedName{
-		Name:      m.Spec.ImagePullSecret,
-		Namespace: m.Namespace,
-	}, pullSecret)
-	if err != nil {
-		sublog.Error(err, "Failed to get secret")
-		return &reconcile.Result{}, err
-	}
-
-	pullSecret.SetNamespace(newNS)
-	pullSecret.SetSelfLink("")
-	pullSecret.SetResourceVersion("")
-	pullSecret.SetUID("")
-
-	unstructuredPullSecret, err := utils.CoreToUnstructured(pullSecret)
-	if err != nil {
-		sublog.Error(err, "Failed to unmarshal into unstructured object")
-		return &reconcile.Result{}, err
-	}
-	utils.AddInstallerLabel(unstructuredPullSecret, m.Name, m.Namespace)
-
-	err = r.client.Get(context.TODO(), types.NamespacedName{
-		Name:      unstructuredPullSecret.GetName(),
-		Namespace: newNS,
-	}, unstructuredPullSecret)
-
-	if err != nil && errors.IsNotFound(err) {
-		sublog.Info(fmt.Sprintf("Creating secret %s in namespace %s", unstructuredPullSecret.GetName(), utils.CertManagerNamespace))
-		err = r.client.Create(context.TODO(), unstructuredPullSecret)
-		if err != nil {
-			sublog.Error(err, "Failed to create secret")
-			return &reconcile.Result{}, err
-		}
-	}
-	return nil, nil
-}
-
 // OverrideImagesFromConfigmap ...
 func (r *ReconcileMultiClusterHub) OverrideImagesFromConfigmap(imageOverrides map[string]string, namespace, configmapName string) (map[string]string, error) {
 	log.Info(fmt.Sprintf("Overriding images from configmap: %s/%s", namespace, configmapName))
@@ -565,22 +589,6 @@ func (r *ReconcileMultiClusterHub) labelDeployments(hub *operatorsv1.MultiCluste
 	return nil
 }
 
-func (r *ReconcileMultiClusterHub) ensureWebhookIsAvailable(mch *operatorsv1.MultiClusterHub) (*reconcile.Result, error) {
-	duration := time.Second * 5
-	if _, ok := mch.Status.Components["cert-manager-webhook-sub"]; !ok {
-		log.Info("Waiting for cert-manager-webhook status")
-		return &reconcile.Result{RequeueAfter: duration}, fmt.Errorf("Waiting for cert-manager-webhook status")
-	}
-
-	component := mch.Status.Components["cert-manager-webhook-sub"]
-	if !((component.Type == "Deployed" || component.Type == "DeployedRelease") && component.Status == "True") {
-		log.Info("Waiting for cert-manager-webhook to be available")
-		return &reconcile.Result{RequeueAfter: duration}, fmt.Errorf("Waiting for cert-manager-webhook to be available")
-	}
-
-	return nil, nil
-}
-
 // ensureSubscriptionOperatorIsRunning verifies that the subscription operator that manages helm subscriptions exists and
 // is running. This validation is only intended to run during upgrade and when run as an OLM managed deployment
 func (r *ReconcileMultiClusterHub) ensureSubscriptionOperatorIsRunning(mch *operatorsv1.MultiClusterHub, allDeps []*appsv1.Deployment) (*reconcile.Result, error) {
@@ -596,7 +604,7 @@ func (r *ReconcileMultiClusterHub) ensureSubscriptionOperatorIsRunning(mch *oper
 	}
 
 	// skip check if not deployed by OLM
-	if !isOLMManaged(selfDeployment) {
+	if !isACMManaged(selfDeployment) {
 		return nil, nil
 	}
 
@@ -625,14 +633,16 @@ func (r *ReconcileMultiClusterHub) ensureSubscriptionOperatorIsRunning(mch *oper
 	}
 }
 
-// isOLMManaged returns whether this application is managed by OLM
-func isOLMManaged(deploy *appsv1.Deployment) bool {
+// isACMManaged returns whether this application is managed by OLM via an ACM subscription
+func isACMManaged(deploy *appsv1.Deployment) bool {
 	labels := deploy.GetLabels()
 	if labels == nil {
 		return false
 	}
-	if _, ok := labels["olm.owner"]; ok {
-		return true
+	if owner, ok := labels["olm.owner"]; ok {
+		if strings.Contains(owner, "advanced-cluster-management") {
+			return true
+		}
 	}
 	return false
 }
@@ -646,3 +656,88 @@ func getDeploymentByName(allDeps []*appsv1.Deployment, desiredDeploy string) (*a
 	}
 	return nil, false
 }
+
+func addProxyEnvVarsToDeployment(dep *appsv1.Deployment) *appsv1.Deployment {
+	proxyEnvVars := []corev1.EnvVar{
+		{
+			Name:  "HTTP_PROXY",
+			Value: os.Getenv("HTTP_PROXY"),
+		},
+		{
+			Name:  "HTTPS_PROXY",
+			Value: os.Getenv("HTTPS_PROXY"),
+		},
+		{
+			Name:  "NO_PROXY",
+			Value: os.Getenv("NO_PROXY"),
+		},
+	}
+	for i := 0; i < len(dep.Spec.Template.Spec.Containers); i++ {
+		dep.Spec.Template.Spec.Containers[i].Env = append(dep.Spec.Template.Spec.Containers[i].Env, proxyEnvVars...)
+	}
+
+	return dep
+}
+
+func addProxyEnvVarsToSub(u *unstructured.Unstructured) *unstructured.Unstructured {
+	path := "spec.packageOverrides[].packageOverrides[].value.hubconfig."
+
+	sub, err := injectMapIntoUnstructured(u.Object, path, map[string]interface{}{
+		"proxyConfigs": map[string]interface{}{
+			"HTTP_PROXY":  os.Getenv("HTTP_PROXY"),
+			"HTTPS_PROXY": os.Getenv("HTTPS_PROXY"),
+			"NO_PROXY":    os.Getenv("NO_PROXY"),
+		},
+	})
+	if err != nil {
+		log.Info(fmt.Sprintf("Error inject proxy environmental variables into '%s' subscription: %s", u.GetName(), err.Error()))
+	}
+	u.Object = sub
+
+	return u
+}
+
+// injects map into an unstructured objects map based off a given path
+func injectMapIntoUnstructured(u map[string]interface{}, path string, mapToInject map[string]interface{}) (map[string]interface{}, error) {
+
+	currentKey := strings.Split(path, ".")[0]
+	isArr := false
+	if strings.HasSuffix(currentKey, "[]") {
+		isArr = true
+	}
+	currentKey = strings.TrimSuffix(currentKey, "[]")
+	if i := strings.Index(path, "."); i >= 0 {
+		// Determine remaining path
+		path = path[(i + 1):]
+		// If Arr, loop through each element of array
+		if isArr {
+			nextMap, ok := u[currentKey].([]map[string]interface{})
+			if !ok {
+				return u, fmt.Errorf("Failed to find key: %s", currentKey)
+			}
+			for i := 0; i < len(nextMap); i++ {
+				_, err := injectMapIntoUnstructured(nextMap[i], path, mapToInject)
+				if err != nil {
+					return u, err
+				}
+			}
+			return u, nil
+		} else {
+			nextMap, ok := u[currentKey].(map[string]interface{})
+			if !ok {
+				return u, fmt.Errorf("Failed to find key: %s", currentKey)
+			}
+			_, err := injectMapIntoUnstructured(nextMap, path, mapToInject)
+			if err != nil {
+				return u, err
+			}
+			return u, nil
+		}
+	} else {
+		for key, val := range mapToInject {
+			u[key] = val
+			break
+		}
+		return u, nil
+	}
+}
