@@ -1,22 +1,27 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package multiclusterhub
 
 import (
 	"context"
 	"fmt"
+	"k8s.io/apimachinery/pkg/api/equality"
+	"k8s.io/apimachinery/pkg/runtime"
+	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	"os"
 	"reflect"
+	"strings"
 	"testing"
 	"time"
 
 	subrelv1 "github.com/open-cluster-management/multicloud-operators-subscription-release/pkg/apis/apps/v1"
-	operatorsv1 "github.com/stolostron/multiclusterhub-operator/pkg/apis/operator/v1"
-	"github.com/stolostron/multiclusterhub-operator/pkg/channel"
-	"github.com/stolostron/multiclusterhub-operator/pkg/foundation"
-	"github.com/stolostron/multiclusterhub-operator/pkg/helmrepo"
-	"github.com/stolostron/multiclusterhub-operator/pkg/manifest"
-	"github.com/stolostron/multiclusterhub-operator/pkg/subscription"
+	operatorsv1 "github.com/open-cluster-management/multiclusterhub-operator/pkg/apis/operator/v1"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/channel"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/foundation"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/helmrepo"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/manifest"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/subscription"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -40,9 +45,9 @@ func Test_ensureDeployment(t *testing.T) {
 	cacheSpec := CacheSpec{
 		IngressDomain: "apps.smart-buck.dev01.red-chesterfield.com",
 		ImageOverrides: map[string]string{
-			"application_ui": "quay.io/stolostron/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
-			"console_api":    "quay.io/stolostron/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
-			"rcm_controller": " quay.io/stolostron/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
+			"application_ui": "quay.io/open-cluster-management/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
+			"console_api":    "quay.io/open-cluster-management/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
+			"rcm_controller": " quay.io/open-cluster-management/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
 		},
 	}
 
@@ -103,40 +108,55 @@ func Test_ensureDeployment(t *testing.T) {
 }
 
 func Test_ensureService(t *testing.T) {
-	r, err := getTestReconciler(full_mch)
-	if err != nil {
-		t.Fatalf("Failed to create test reconciler")
-	}
+	expectedWebhookService := foundation.WebhookService(full_mch)
+	expectedWebhookService.SetAnnotations(map[string]string{"service.beta.openshift.io/serving-cert-secret-name": "test"})
 
 	tests := []struct {
-		Name    string
-		MCH     *operatorsv1.MultiClusterHub
-		Service *corev1.Service
-		Result  error
+		Name           string
+		MCH            *operatorsv1.MultiClusterHub
+		ExistedService *corev1.Service
+		ExpectService  *corev1.Service
+		Result         error
 	}{
 		{
-			Name:    "Test: ensureService - Multiclusterhub-repo",
-			MCH:     full_mch,
-			Service: helmrepo.Service(full_mch),
-			Result:  nil,
+			Name:          "Test: ensureService - Multiclusterhub-repo",
+			MCH:           full_mch,
+			ExpectService: helmrepo.Service(full_mch),
+			Result:        nil,
 		},
 		{
-			Name:    "Test: ensureService - Webhook",
-			MCH:     full_mch,
-			Service: foundation.WebhookService(full_mch),
-			Result:  nil,
+			Name:          "Test: ensureService - Webhook",
+			MCH:           full_mch,
+			ExpectService: foundation.WebhookService(full_mch),
+			Result:        nil,
 		},
 		{
-			Name:    "Test: ensureService - Empty service",
-			MCH:     full_mch,
-			Service: &corev1.Service{},
-			Result:  errors.NewInvalid(kind("Test"), "", nil),
+			Name:           "Test: ensureService - update Webhook Service",
+			MCH:            full_mch,
+			ExistedService: foundation.WebhookService(full_mch),
+			ExpectService:  expectedWebhookService,
+			Result:         nil,
+		},
+		{
+			Name:          "Test: ensureService - Empty service",
+			MCH:           full_mch,
+			ExpectService: &corev1.Service{},
+			Result:        errors.NewInvalid(kind("Test"), "", nil),
 		},
 	}
 
 	for _, tt := range tests {
 		t.Run(tt.Name, func(t *testing.T) {
-			_, err = r.ensureService(tt.MCH, tt.Service)
+			objs := []runtime.Object{tt.MCH}
+			if tt.ExistedService != nil {
+				objs = append(objs, tt.ExistedService)
+			}
+			r, err := getTestReconcilerWithObjs(objs)
+			if err != nil {
+				t.Fatalf("Failed to create test reconciler")
+			}
+
+			_, err = r.ensureService(tt.MCH, tt.ExpectService)
 
 			if tt.Result != nil {
 				// Check if error matches desired error
@@ -150,12 +170,88 @@ func Test_ensureService(t *testing.T) {
 
 				service := &corev1.Service{}
 				err = r.client.Get(context.TODO(), types.NamespacedName{
-					Name:      tt.Service.Name,
-					Namespace: tt.Service.Namespace,
+					Name:      tt.ExpectService.Name,
+					Namespace: tt.ExpectService.Namespace,
 				}, service)
 				if err != tt.Result {
-					t.Fatalf("Could not find created '%s' service: %s", tt.Service.Name, err.Error())
+					t.Fatalf("Could not find created '%s' service: %s", tt.ExpectService.Name, err.Error())
 				}
+				if tt.ExistedService != nil {
+					if !equality.Semantic.DeepEqual(service.Annotations, tt.ExpectService.Annotations) &&
+						!equality.Semantic.DeepEqual(service.Labels, tt.ExpectService.Labels) {
+						t.Fatalf("could not create/upgrade the right service")
+					}
+				}
+
+			}
+		})
+	}
+}
+
+func Test_ensureAPIService(t *testing.T) {
+	expectedAPIService := foundation.OCMProxyAPIService(full_mch)
+	expectedAPIService.SetAnnotations(map[string]string{"service.beta.openshift.io/serving-cert-secret-name": "test"})
+
+	tests := []struct {
+		Name           string
+		MCH            *operatorsv1.MultiClusterHub
+		ExistedService *apiregistrationv1.APIService
+		ExpectService  *apiregistrationv1.APIService
+		Result         error
+	}{
+		{
+			Name:          "Test: ensureAPIService - OCMProxyServer",
+			MCH:           full_mch,
+			ExpectService: foundation.OCMProxyAPIService(full_mch),
+			Result:        nil,
+		},
+		{
+			Name:           "Test: ensureService - update Webhook Service",
+			MCH:            full_mch,
+			ExistedService: foundation.OCMProxyAPIService(full_mch),
+			ExpectService:  expectedAPIService,
+			Result:         nil,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.Name, func(t *testing.T) {
+			objs := []runtime.Object{tt.MCH}
+			if tt.ExistedService != nil {
+				objs = append(objs, tt.ExistedService)
+			}
+			r, err := getTestReconcilerWithObjs(objs)
+			if err != nil {
+				t.Fatalf("Failed to create test reconciler")
+			}
+
+			_, err = r.ensureAPIService(tt.MCH, tt.ExpectService)
+
+			if tt.Result != nil {
+				// Check if error matches desired error
+				if errors.ReasonForError(err) != errors.ReasonForError(tt.Result) {
+					t.Fatalf("ensureAPIService() error = %v, wantErr %v", err, tt.Result)
+				}
+			} else {
+				if err != nil {
+					t.Fatalf("ensureAPIService() error = %v, wantErr %v", err, tt.Result)
+				}
+
+				service := &apiregistrationv1.APIService{}
+				err = r.client.Get(context.TODO(), types.NamespacedName{
+					Name:      tt.ExpectService.Name,
+					Namespace: tt.ExpectService.Namespace,
+				}, service)
+				if err != tt.Result {
+					t.Fatalf("Could not find created '%s' APIService: %s", tt.ExpectService.Name, err.Error())
+				}
+				if tt.ExistedService != nil {
+					if !equality.Semantic.DeepEqual(service.Annotations, tt.ExpectService.Annotations) &&
+						!equality.Semantic.DeepEqual(service.Labels, tt.ExpectService.Labels) {
+						t.Fatalf("could not create/upgrade the right APIService")
+					}
+				}
+
 			}
 		})
 	}
@@ -210,9 +306,9 @@ func Test_ensureSubscription(t *testing.T) {
 	cacheSpec := CacheSpec{
 		IngressDomain: "apps.smart-buck.dev01.red-chesterfield.com",
 		ImageOverrides: map[string]string{
-			"application_ui": "quay.io/stolostron/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
-			"console_api":    "quay.io/stolostron/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
-			"rcm_controller": " quay.io/stolostron/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
+			"application_ui": "quay.io/open-cluster-management/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
+			"console_api":    "quay.io/open-cluster-management/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
+			"rcm_controller": " quay.io/open-cluster-management/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
 		},
 	}
 
@@ -222,24 +318,6 @@ func Test_ensureSubscription(t *testing.T) {
 		Subscription *unstructured.Unstructured
 		Result       error
 	}{
-		{
-			Name:         "Test: ensureSubscription - Cert-manager",
-			MCH:          full_mch,
-			Subscription: subscription.CertManager(full_mch, cacheSpec.ImageOverrides),
-			Result:       nil,
-		},
-		{
-			Name:         "Test: ensureSubscription - Cert-webhook",
-			MCH:          full_mch,
-			Subscription: subscription.CertWebhook(full_mch, cacheSpec.ImageOverrides),
-			Result:       nil,
-		},
-		{
-			Name:         "Test: ensureSubscription - Config-watcher",
-			MCH:          full_mch,
-			Subscription: subscription.ConfigWatcher(full_mch, cacheSpec.ImageOverrides),
-			Result:       nil,
-		},
 		{
 			Name:         "Test: ensureSubscription - Management-ingress",
 			MCH:          full_mch,
@@ -258,6 +336,18 @@ func Test_ensureSubscription(t *testing.T) {
 			Subscription: subscription.Console(full_mch, cacheSpec.ImageOverrides, cacheSpec.IngressDomain),
 			Result:       nil,
 		},
+		{
+			Name:         "Test: ensureSubscription - Insights",
+			MCH:          full_mch,
+			Subscription: subscription.Insights(full_mch, cacheSpec.ImageOverrides, cacheSpec.IngressDomain),
+			Result:       nil,
+		},
+		{
+			Name:         "Test: ensureSubscription - Discovery",
+			MCH:          full_mch,
+			Subscription: subscription.Discovery(full_mch, cacheSpec.ImageOverrides),
+			Result:       nil,
+		},
 		{
 			Name:         "Test: ensureSubscription - GRC",
 			MCH:          full_mch,
@@ -271,9 +361,9 @@ func Test_ensureSubscription(t *testing.T) {
 			Result:       nil,
 		},
 		{
-			Name:         "Test: ensureSubscription - RCM",
+			Name:         "Test: ensureSubscription - cluster-lifecycle",
 			MCH:          full_mch,
-			Subscription: subscription.RCM(full_mch, cacheSpec.ImageOverrides),
+			Subscription: subscription.ClusterLifecycle(full_mch, cacheSpec.ImageOverrides),
 			Result:       nil,
 		},
 		{
@@ -282,12 +372,6 @@ func Test_ensureSubscription(t *testing.T) {
 			Subscription: subscription.Search(full_mch, cacheSpec.ImageOverrides),
 			Result:       nil,
 		},
-		{
-			Name:         "Test: ensureSubscription - Topology",
-			MCH:          full_mch,
-			Subscription: subscription.Topology(full_mch, cacheSpec.ImageOverrides),
-			Result:       nil,
-		},
 		{
 			Name:         "Test: ensureSubscription - Empty Sub",
 			MCH:          full_mch,
@@ -315,7 +399,7 @@ func Test_ensureUnstructuredResource(t *testing.T) {
 	}
 
 	imageOverrides := map[string]string{
-		"registration": "quay.io/stolostron/registration@sha256:fe95bca419976ca8ffe608bc66afcead6ef333b863f22be55df57c89ded75dda",
+		"registration": "quay.io/open-cluster-management/registration@sha256:fe95bca419976ca8ffe608bc66afcead6ef333b863f22be55df57c89ded75dda",
 	}
 
 	tests := []struct {
@@ -384,8 +468,8 @@ func Test_OverrideImagesFromConfigmap(t *testing.T) {
 					"overrides.json:": `[
 						{
 						  "image-name": "multiclusterhub-repo",
-						  "image-tag": "2.2.12-test",
-						  "image-remote": "quay.io/stolostron",
+						  "image-tag": "2.3.4-test",
+						  "image-remote": "quay.io/open-cluster-management",
 						  "image-key": "multiclusterhub_repo"
 						}
 					  ]`,
@@ -393,8 +477,8 @@ func Test_OverrideImagesFromConfigmap(t *testing.T) {
 			},
 			ManifestImage: manifest.ManifestImage{
 				ImageKey:    "multiclusterhub_repo",
-				ImageRemote: "quay.io/stolostron",
-				ImageTag:    "2.2.12-test",
+				ImageRemote: "quay.io/open-cluster-management",
+				ImageTag:    "2.3.4-test",
 				ImageName:   "multiclusterhub-repo",
 			},
 			Result: nil,
@@ -413,7 +497,7 @@ func Test_OverrideImagesFromConfigmap(t *testing.T) {
 						{
 						  "image-name": "non-existent-image",
 						  "image-digest": "sha256:e728a4cdf4a11b78b927b7b280d75babca7b3880450fbf190d80b194f7d064b6",
-						  "image-remote": "quay.io/stolostron",
+						  "image-remote": "quay.io/open-cluster-management",
 						  "image-key": "non_existent_image"
 						}
 					  ]`,
@@ -421,7 +505,7 @@ func Test_OverrideImagesFromConfigmap(t *testing.T) {
 			},
 			ManifestImage: manifest.ManifestImage{
 				ImageKey:    "non_existent_image",
-				ImageRemote: "quay.io/stolostron",
+				ImageRemote: "quay.io/open-cluster-management",
 				ImageDigest: "sha256:e728a4cdf4a11b78b927b7b280d75babca7b3880450fbf190d80b194f7d064b6",
 				ImageName:   "non-existent-image",
 			},
@@ -465,11 +549,11 @@ func Test_maintainImageManifestConfigmap(t *testing.T) {
 
 	r.CacheSpec = CacheSpec{
 		ImageOverrides: map[string]string{
-			"application_ui": "quay.io/stolostron/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
-			"console_api":    "quay.io/stolostron/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
-			"rcm_controller": " quay.io/stolostron/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
+			"application_ui": "quay.io/open-cluster-management/application-ui@sha256:c740fc7bac067f003145ab909504287360564016b7a4a51b7ad4987aca123ac1",
+			"console_api":    "quay.io/open-cluster-management/console-api@sha256:3ef1043b4e61a09b07ff37f9ad8fc6e707af9813936cf2c0d52f2fa0e489c75f",
+			"rcm_controller": " quay.io/open-cluster-management/rcm-controller@sha256:8fab4d788241bf364dbc1b8c1ea5ccf18d3145a640dbd456b0dc7ba204e36819",
 		},
-		ManifestVersion: "2.2.12",
+		ManifestVersion: "2.3.4",
 	}
 
 	configmapName := fmt.Sprintf("mch-image-manifest-%s", r.CacheSpec.ManifestVersion)
@@ -630,7 +714,7 @@ func Test_addInstallerLabel(t *testing.T) {
 func Test_getAppSubOwnedHelmReleases(t *testing.T) {
 	appsubs := []types.NamespacedName{
 		{Name: "search-prod-sub", Namespace: "default"},
-		{Name: "topology-sub", Namespace: "default"},
+		{Name: "console-sub", Namespace: "default"},
 	}
 
 	tests := []struct {
@@ -663,7 +747,7 @@ func Test_getAppSubOwnedHelmReleases(t *testing.T) {
 						{
 							APIVersion: "apps.open-cluster-management.io/v1",
 							Kind:       "Subscription",
-							Name:       "topology-sub",
+							Name:       "console-sub",
 						},
 					},
 				}},
@@ -675,7 +759,7 @@ func Test_getAppSubOwnedHelmReleases(t *testing.T) {
 						{
 							APIVersion: "apps.open-cluster-management.io/v1",
 							Kind:       "Subscription",
-							Name:       "topology-sub",
+							Name:       "console-sub",
 						},
 					},
 				}},
@@ -734,59 +818,6 @@ func Test_getHelmReleaseOwnedDeployments(t *testing.T) {
 	}
 }
 
-func Test_ensureWebhookIsAvailable(t *testing.T) {
-	availableWebhook := full_mch.DeepCopy()
-	unavailableWebhook := full_mch.DeepCopy()
-
-	statusConditionAvailable := operatorsv1.StatusCondition{
-		Type:   "Deployed",
-		Status: "True",
-	}
-	statusConditionUnavailable := operatorsv1.StatusCondition{
-		Type:   "Error",
-		Status: "False",
-	}
-	availableWebhook.Status.Components = make(map[string]operatorsv1.StatusCondition)
-	unavailableWebhook.Status.Components = make(map[string]operatorsv1.StatusCondition)
-
-	availableWebhook.Status.Components["cert-manager-webhook-sub"] = statusConditionAvailable
-	unavailableWebhook.Status.Components["cert-manager-webhook-sub"] = statusConditionUnavailable
-
-	tests := []struct {
-		name   string
-		mch    *operatorsv1.MultiClusterHub
-		result error
-	}{
-		{
-			name:   "No status",
-			mch:    full_mch,
-			result: fmt.Errorf("Waiting for cert-manager-webhook status"),
-		},
-		{
-			name:   "Webhook is Unavailable",
-			mch:    unavailableWebhook,
-			result: fmt.Errorf("Waiting for cert-manager-webhook to be available"),
-		},
-		{
-			name:   "Webhook is available",
-			mch:    availableWebhook,
-			result: nil,
-		},
-	}
-	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
-			r, err := getTestReconciler(full_mch)
-			if err != nil {
-				t.Fatalf("Failed to create test reconciler")
-			}
-			_, err = r.ensureWebhookIsAvailable(tt.mch)
-			if !errorEquals(err, tt.result) {
-				t.Fatalf("Improper webhook status")
-			}
-		})
-	}
-}
-
 func Test_getDeploymentByName(t *testing.T) {
 	foo := &appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "foo"}}
 	bar := &appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "bar"}}
@@ -826,7 +857,7 @@ func Test_getDeploymentByName(t *testing.T) {
 	}
 }
 
-func Test_isOLMManaged(t *testing.T) {
+func Test_isACMManaged(t *testing.T) {
 	foo := &appsv1.Deployment{ObjectMeta: metav1.ObjectMeta{Name: "foo"}}
 	bar := &appsv1.Deployment{
 		ObjectMeta: metav1.ObjectMeta{
@@ -840,6 +871,12 @@ func Test_isOLMManaged(t *testing.T) {
 			Labels: map[string]string{"olm.owner": "foobar"},
 		},
 	}
+	acmDeploy := &appsv1.Deployment{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   "olm",
+			Labels: map[string]string{"olm.owner": "advanced-cluster-management.v2.3.4"},
+		},
+	}
 
 	tests := []struct {
 		name   string
@@ -857,14 +894,19 @@ func Test_isOLMManaged(t *testing.T) {
 			want:   false,
 		},
 		{
-			name:   "OLM installed",
+			name:   "OLM managed",
 			deploy: olmDeploy,
+			want:   false,
+		},
+		{
+			name:   "OLM managed by ACM",
+			deploy: acmDeploy,
 			want:   true,
 		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			if got := isOLMManaged(tt.deploy); got != tt.want {
+			if got := isACMManaged(tt.deploy); got != tt.want {
 				t.Errorf("isOLMManaged() = %v, want %v", got, tt.want)
 			}
 		})
@@ -958,3 +1000,154 @@ func TestReconcileMultiClusterHub_ensureSubscriptionOperatorIsRunning(t *testing
 	}
 
 }
+
+func TestAddProxyEnvVarsToDeployment(t *testing.T) {
+
+	cacheSpec := CacheSpec{
+		IngressDomain:  "",
+		ImageOverrides: map[string]string{},
+	}
+
+	tests := []struct {
+		deploy       *appsv1.Deployment
+		envVars      map[string]string
+		expectedVars map[string]string
+	}{
+		{
+			deploy: helmrepo.Deployment(full_mch, cacheSpec.ImageOverrides),
+			envVars: map[string]string{
+				"HTTP_PROXY":  "test_http_proxy",
+				"HTTPS_PROXY": "test_https_proxy",
+				"NO_PROXY":    "no_proxy",
+			},
+			expectedVars: map[string]string{
+				"HTTP_PROXY":  "test_http_proxy",
+				"HTTPS_PROXY": "test_https_proxy",
+				"NO_PROXY":    "no_proxy",
+			},
+		}, {
+			deploy: foundation.WebhookDeployment(full_mch, cacheSpec.ImageOverrides),
+			envVars: map[string]string{
+				"HTTP_PROXY":  "test_http_proxy",
+				"HTTPS_PROXY": "test_https_proxy",
+			},
+			expectedVars: map[string]string{
+				"HTTP_PROXY":  "test_http_proxy",
+				"HTTPS_PROXY": "test_https_proxy",
+				"NO_PROXY":    "",
+			},
+		},
+		{
+			deploy: foundation.OCMProxyServerDeployment(full_mch, cacheSpec.ImageOverrides),
+			envVars: map[string]string{
+				"HTTP_PROXY": "test_http_proxy",
+			},
+			expectedVars: map[string]string{
+				"HTTP_PROXY": "test_http_proxy",
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		for key, value := range tt.envVars {
+			os.Setenv(key, value)
+		}
+		dep := addProxyEnvVarsToDeployment(tt.deploy)
+		containers := dep.Spec.Template.Spec.Containers
+		for _, container := range containers {
+			for expectedKey, expectedVal := range tt.expectedVars {
+				found := false
+				for _, val := range container.Env {
+					if expectedKey == val.Name && expectedVal == val.Value {
+						found = true
+					}
+				}
+				if !found {
+					t.Logf("Expected Variables: %+v\n", tt.expectedVars)
+					t.Logf("Found Variables: %+v\n", container.Env)
+					t.Fatalf("Expected proxy overrides not found")
+				}
+			}
+		}
+		for key := range tt.envVars {
+			os.Unsetenv(key)
+		}
+	}
+}
+
+func TestAddProxyEnvVarsToSub(t *testing.T) {
+
+	cacheSpec := CacheSpec{
+		IngressDomain:  "",
+		ImageOverrides: map[string]string{},
+	}
+
+	tests := []struct {
+		sub          *unstructured.Unstructured
+		envVars      map[string]string
+		expectedVars map[string]string
+	}{
+		{
+			sub:          subscription.ManagementIngress(full_mch, cacheSpec.ImageOverrides, cacheSpec.IngressDomain),
+			envVars:      map[string]string{},
+			expectedVars: map[string]string{},
+		},
+	}
+
+	for _, tt := range tests {
+		for key, value := range tt.envVars {
+			os.Setenv(key, value)
+		}
+		sub := addProxyEnvVarsToSub(tt.sub)
+
+		path := "spec.packageOverrides[].packageOverrides[].value.hubconfig.proxyConfigs."
+		proxyOverrides, err := getMapFromUnstructured(sub.Object, path)
+		if err != nil {
+			t.Fatalf("Failed to find proxyConfigs: %s", err.Error())
+		}
+
+		for expectedKey, expectedVal := range tt.expectedVars {
+			found := false
+			for key, val := range proxyOverrides {
+				if expectedKey == key && expectedVal == val.(string) {
+					found = true
+				}
+			}
+			if !found {
+				t.Logf("Expected Variables: %+v\n", tt.expectedVars)
+				t.Logf("Found Variables: %+v\n", proxyOverrides)
+				t.Fatalf("Expected proxy overrides not found")
+			}
+		}
+	}
+}
+
+// Currently function has limitation of only retrieving the first map
+func getMapFromUnstructured(u map[string]interface{}, path string) (map[string]interface{}, error) {
+	currentKey := strings.Split(path, ".")[0]
+	isArr := false
+	if strings.HasSuffix(currentKey, "[]") {
+		isArr = true
+	}
+	currentKey = strings.TrimSuffix(currentKey, "[]")
+	if i := strings.Index(path, "."); i >= 0 {
+		// Determine remaining path
+		path = path[(i + 1):]
+		// If Arr, loop through each element of array
+		if isArr {
+			nextMap, ok := u[currentKey].([]map[string]interface{})
+			if !ok {
+				return u, fmt.Errorf("Failed to find key: %s", currentKey)
+			}
+			return getMapFromUnstructured(nextMap[0], path)
+		} else {
+			nextMap, ok := u[currentKey].(map[string]interface{})
+			if !ok {
+				return u, fmt.Errorf("Failed to find key: %s", currentKey)
+			}
+			return getMapFromUnstructured(nextMap, path)
+		}
+	} else {
+		return u, nil
+	}
+}
