@@ -1,18 +1,21 @@
-// Copyright (c) 2020 Red Hat, Inc.
+// Copyright (c) 2021 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package multiclusterhub
 
 import (
 	"context"
 	"fmt"
+	"os"
+	"strings"
 	"time"
 
+	hive "github.com/openshift/hive/pkg/apis/hive/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/util/workqueue"
 	"sigs.k8s.io/controller-runtime/pkg/client"
@@ -26,19 +29,21 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
 	"github.com/go-logr/logr"
+	clustermanager "github.com/open-cluster-management/api/operator/v1"
 	appsubv1 "github.com/open-cluster-management/multicloud-operators-subscription/pkg/apis/apps/v1"
+	operatorsv1 "github.com/open-cluster-management/multiclusterhub-operator/pkg/apis/operator/v1"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/channel"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/deploying"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/foundation"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/helmrepo"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/imageoverrides"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/manifest"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/predicate"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/rendering"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/subscription"
+	"github.com/open-cluster-management/multiclusterhub-operator/pkg/utils"
+	"github.com/open-cluster-management/multiclusterhub-operator/version"
 	netv1 "github.com/openshift/api/config/v1"
-	operatorsv1 "github.com/stolostron/multiclusterhub-operator/pkg/apis/operator/v1"
-	"github.com/stolostron/multiclusterhub-operator/pkg/channel"
-	"github.com/stolostron/multiclusterhub-operator/pkg/deploying"
-	"github.com/stolostron/multiclusterhub-operator/pkg/foundation"
-	"github.com/stolostron/multiclusterhub-operator/pkg/helmrepo"
-	"github.com/stolostron/multiclusterhub-operator/pkg/manifest"
-	"github.com/stolostron/multiclusterhub-operator/pkg/predicate"
-	"github.com/stolostron/multiclusterhub-operator/pkg/rendering"
-	"github.com/stolostron/multiclusterhub-operator/pkg/subscription"
-	"github.com/stolostron/multiclusterhub-operator/pkg/utils"
-	"github.com/stolostron/multiclusterhub-operator/version"
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 )
 
@@ -120,6 +125,47 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
+	err = c.Watch(
+		&source.Kind{Type: &hive.HiveConfig{}},
+		&handler.Funcs{
+			DeleteFunc: func(e event.DeleteEvent, q workqueue.RateLimitingInterface) {
+				labels := e.Meta.GetLabels()
+				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+					Name:      labels["installer.name"],
+					Namespace: labels["installer.namespace"],
+				}})
+			},
+		},
+		predicate.InstallerLabelPredicate{},
+	)
+	if err != nil {
+		return err
+	}
+
+	err = c.Watch(
+		&source.Kind{Type: &clustermanager.ClusterManager{}},
+		&handler.Funcs{
+			DeleteFunc: func(e event.DeleteEvent, q workqueue.RateLimitingInterface) {
+				labels := e.Meta.GetLabels()
+				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+					Name:      labels["installer.name"],
+					Namespace: labels["installer.namespace"],
+				}})
+			},
+			UpdateFunc: func(e event.UpdateEvent, q workqueue.RateLimitingInterface) {
+				labels := e.MetaOld.GetLabels()
+				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{
+					Name:      labels["installer.name"],
+					Namespace: labels["installer.namespace"],
+				}})
+			},
+		},
+		predicate.InstallerLabelPredicate{},
+	)
+	if err != nil {
+		return err
+	}
+
 	err = c.Watch(
 		&source.Kind{Type: &appsv1.Deployment{}},
 		&handler.EnqueueRequestsFromMapFunc{
@@ -251,10 +297,21 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	}
 
 	// Read image overrides
-	imageOverrides, err := manifest.GetImageOverrides(multiClusterHub)
-	if err != nil {
-		reqLogger.Error(err, "Could not get map of image overrides")
-		return reconcile.Result{}, err
+	// First, attempt to read image overrides from environmental variables
+	imageOverrides := imageoverrides.GetImageOverrides()
+	if len(imageOverrides) == 0 {
+		// If imageoverrides are not set from environmental variables, read from manifest
+		reqLogger.Info("Image Overrides not set from environment variables. Checking for overrides in manifest")
+		imageOverrides, err = manifest.GetImageOverrides(multiClusterHub)
+		if err != nil {
+			reqLogger.Error(err, "Could not get map of image overrides")
+			return reconcile.Result{}, err
+		}
+	}
+
+	if imageRepo := utils.GetImageRepository(multiClusterHub); imageRepo != "" {
+		reqLogger.Info(fmt.Sprintf("Overriding Image Repository from annotation 'mch-imageRepository': %s", imageRepo))
+		imageOverrides = utils.OverrideImageRepository(imageOverrides, imageRepo)
 	}
 
 	// Check for developer overrides
@@ -267,9 +324,7 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	}
 	r.CacheSpec.ImageOverrides = imageOverrides
 	r.CacheSpec.ManifestVersion = version.Version
-	r.CacheSpec.ImageOverrideType = manifest.GetImageOverrideType(multiClusterHub)
 	r.CacheSpec.ImageRepository = utils.GetImageRepository(multiClusterHub)
-	r.CacheSpec.ImageSuffix = utils.GetImageSuffix(multiClusterHub)
 	r.CacheSpec.ImageOverridesCM = utils.GetImageOverridesConfigmap(multiClusterHub)
 
 	err = r.maintainImageManifestConfigmap(multiClusterHub)
@@ -278,13 +333,13 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 		return reconcile.Result{}, err
 	}
 
-	UpgradeHackRequired, err := r.UpgradeHubSelfMgmtHackRequired(multiClusterHub)
+	CustomUpgradeRequired, err := r.CustomSelfMgmtHubUpgradeRequired(multiClusterHub)
 	if err != nil {
 		reqLogger.Error(err, "Error determining if upgrade specific logic is required")
 		return reconcile.Result{}, err
 	}
 
-	if UpgradeHackRequired {
+	if CustomUpgradeRequired {
 		result, err = r.BeginEnsuringHubIsUpgradeable(multiClusterHub)
 		if err != nil {
 			log.Info(fmt.Sprintf("Error starting to ensure local-cluster hub is upgradeable: %s", err.Error()))
@@ -312,26 +367,13 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	}
 
 	// Render CRD templates
-	crdRenderer, err := rendering.NewCRDRenderer(multiClusterHub)
+	err = r.installCRDs(reqLogger, multiClusterHub)
 	if err != nil {
-		reqLogger.Error(err, "Failed to read CRD templates")
 		return reconcile.Result{}, err
 	}
-	crdResources, err := crdRenderer.Render()
-	if err != nil {
-		reqLogger.Error(err, "Failed to render CRD templates")
-		return reconcile.Result{}, err
-	}
-	for _, crd := range crdResources {
-		err, ok := deploying.Deploy(r.client, crd)
-		if err != nil {
-			reqLogger.Error(err, fmt.Sprintf("Failed to deploy %s %s", crd.GetKind(), crd.GetName()))
-			return reconcile.Result{}, err
-		}
-		if ok {
-			condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionTrue, NewComponentReason, "Created new resource")
-			SetHubCondition(&multiClusterHub.Status, *condition)
-		}
+
+	if utils.ProxyEnvVarsAreSet() {
+		log.Info(fmt.Sprintf("Proxy configuration environment variables are set. HTTP_PROXY: %s, HTTPS_PROXY: %s, NO_PROXY: %s", os.Getenv("HTTP_PROXY"), os.Getenv("HTTPS_PROXY"), os.Getenv("NO_PROXY")))
 	}
 
 	result, err = r.ensureDeployment(multiClusterHub, helmrepo.Deployment(multiClusterHub, r.CacheSpec.ImageOverrides))
@@ -349,44 +391,6 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 		return *result, err
 	}
 
-	if multiClusterHub.Spec.SeparateCertificateManagement && multiClusterHub.Spec.ImagePullSecret != "" {
-		result, err = r.copyPullSecret(multiClusterHub, utils.CertManagerNamespace)
-		if result != nil {
-			return *result, err
-		}
-	}
-
-	result, err = r.ensureSubscription(multiClusterHub, subscription.CertManager(multiClusterHub, r.CacheSpec.ImageOverrides))
-	if result != nil {
-		return *result, err
-	}
-
-	certGV := schema.GroupVersion{Group: "certmanager.k8s.io", Version: "v1alpha1"}
-	// Skip wait for API to be ready on unit test
-	if !utils.IsUnitTest() {
-		// condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionTrue, CertManagerReason, "Waiting for cert manager CRDs")
-		// SetHubCondition(&multiClusterHub.Status, *condition)
-		result, err = r.apiReady(certGV)
-		if result != nil {
-			return *result, err
-		}
-	}
-
-	result, err = r.ensureSubscription(multiClusterHub, subscription.CertWebhook(multiClusterHub, r.CacheSpec.ImageOverrides))
-	if result != nil {
-		return *result, err
-	}
-
-	result, _ = r.ensureWebhookIsAvailable(multiClusterHub)
-	if result != nil {
-		return *result, nil
-	}
-
-	result, err = r.ensureSubscription(multiClusterHub, subscription.ConfigWatcher(multiClusterHub, r.CacheSpec.ImageOverrides))
-	if result != nil {
-		return *result, err
-	}
-
 	result, err = r.ingressDomain(multiClusterHub)
 	if result != nil {
 		return *result, err
@@ -417,6 +421,22 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 		}
 	}
 
+	result, err = r.ensureDeployment(multiClusterHub, foundation.WebhookDeployment(multiClusterHub, r.CacheSpec.ImageOverrides))
+	if result != nil {
+		return *result, err
+	}
+
+	result, err = r.ensureService(multiClusterHub, foundation.WebhookService(multiClusterHub))
+	if result != nil {
+		return *result, err
+	}
+
+	// Wait for ocm-webhook to be fully available before applying rest of subscriptions
+	if !(multiClusterHub.Status.Components["ocm-webhook"].Type == "Available" && multiClusterHub.Status.Components["ocm-webhook"].Status == metav1.ConditionTrue) {
+		reqLogger.Info(fmt.Sprintf("Waiting for component 'ocm-webhook' to be available"))
+		return reconcile.Result{RequeueAfter: resyncPeriod}, nil
+	}
+
 	// Install the rest of the subscriptions in no particular order
 	result, err = r.ensureSubscription(multiClusterHub, subscription.ManagementIngress(multiClusterHub, r.CacheSpec.ImageOverrides, r.CacheSpec.IngressDomain))
 	if result != nil {
@@ -430,6 +450,14 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	if result != nil {
 		return *result, err
 	}
+	result, err = r.ensureSubscription(multiClusterHub, subscription.Insights(multiClusterHub, r.CacheSpec.ImageOverrides, r.CacheSpec.IngressDomain))
+	if result != nil {
+		return *result, err
+	}
+	result, err = r.ensureSubscription(multiClusterHub, subscription.Discovery(multiClusterHub, r.CacheSpec.ImageOverrides))
+	if result != nil {
+		return *result, err
+	}
 	result, err = r.ensureSubscription(multiClusterHub, subscription.GRC(multiClusterHub, r.CacheSpec.ImageOverrides))
 	if result != nil {
 		return *result, err
@@ -438,7 +466,7 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	if result != nil {
 		return *result, err
 	}
-	result, err = r.ensureSubscription(multiClusterHub, subscription.RCM(multiClusterHub, r.CacheSpec.ImageOverrides))
+	result, err = r.ensureSubscription(multiClusterHub, subscription.ClusterLifecycle(multiClusterHub, r.CacheSpec.ImageOverrides))
 	if result != nil {
 		return *result, err
 	}
@@ -446,16 +474,7 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 	if result != nil {
 		return *result, err
 	}
-	result, err = r.ensureSubscription(multiClusterHub, subscription.Topology(multiClusterHub, r.CacheSpec.ImageOverrides))
-	if result != nil {
-		return *result, err
-	}
-	result, err = r.ensureDeployment(multiClusterHub, foundation.WebhookDeployment(multiClusterHub, r.CacheSpec.ImageOverrides))
-	if result != nil {
-		return *result, err
-	}
-
-	result, err = r.ensureService(multiClusterHub, foundation.WebhookService(multiClusterHub))
+	result, err = r.ensureSubscription(multiClusterHub, subscription.AssistedService(multiClusterHub, r.CacheSpec.ImageOverrides))
 	if result != nil {
 		return *result, err
 	}
@@ -472,6 +491,24 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 		return *result, err
 	}
 
+	// OCM proxy apiService
+	result, err = r.ensureAPIService(multiClusterHub, foundation.OCMProxyAPIService(multiClusterHub))
+	if result != nil {
+		return *result, err
+	}
+
+	// OCM clusterView v1 apiService
+	result, err = r.ensureAPIService(multiClusterHub, foundation.OCMClusterViewV1APIService(multiClusterHub))
+	if result != nil {
+		return *result, err
+	}
+
+	// OCM clusterView v1alpha1 apiService
+	result, err = r.ensureAPIService(multiClusterHub, foundation.OCMClusterViewV1alpha1APIService(multiClusterHub))
+	if result != nil {
+		return *result, err
+	}
+
 	//OCM controller deployment
 	result, err = r.ensureDeployment(multiClusterHub, foundation.OCMControllerDeployment(multiClusterHub, r.CacheSpec.ImageOverrides))
 	if result != nil {
@@ -495,6 +532,14 @@ func (r *ReconcileMultiClusterHub) Reconcile(request reconcile.Request) (retQueu
 		}
 	}
 
+	// Cleanup unused resources once components up-to-date
+	if r.ComponentsAreRunning(multiClusterHub) {
+		result, err = r.ensureRemovalsGone(multiClusterHub)
+		if result != nil {
+			return *result, err
+		}
+	}
+
 	return retQueue, retError
 }
 
@@ -569,6 +614,9 @@ func (r *ReconcileMultiClusterHub) finalizeHub(reqLogger logr.Logger, m *operato
 	if err := r.cleanupMutatingWebhooks(reqLogger, m); err != nil {
 		return err
 	}
+	if err := r.cleanupValidatingWebhooks(reqLogger, m); err != nil {
+		return err
+	}
 	if err := r.cleanupCRDs(reqLogger, m); err != nil {
 		return err
 	}
@@ -598,6 +646,38 @@ func (r *ReconcileMultiClusterHub) addFinalizer(reqLogger logr.Logger, m *operat
 	return nil
 }
 
+func (r *ReconcileMultiClusterHub) installCRDs(reqLogger logr.Logger, m *operatorsv1.MultiClusterHub) error {
+	crdRenderer, err := rendering.NewCRDRenderer(m)
+	if err != nil {
+		condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionFalse, ResourceRenderReason, fmt.Sprintf("Error creating CRD renderer: %s", err.Error()))
+		SetHubCondition(&m.Status, *condition)
+		return fmt.Errorf("failed to setup CRD templates: %w", err)
+	}
+	crdResources, errs := crdRenderer.Render()
+	if errs != nil && len(errs) > 0 {
+		message := mergeErrors(errs)
+		condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionFalse, ResourceRenderReason, fmt.Sprintf("Error rendering CRD templates: %s", message))
+		SetHubCondition(&m.Status, *condition)
+		return fmt.Errorf("failed to render CRD templates: %s", message)
+	}
+
+	for _, crd := range crdResources {
+		err, ok := deploying.Deploy(r.client, crd)
+		if err != nil {
+			message := fmt.Sprintf("Failed to deploy %s %s", crd.GetKind(), crd.GetName())
+			reqLogger.Error(err, message)
+			condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionFalse, DeployFailedReason, message)
+			SetHubCondition(&m.Status, *condition)
+			return err
+		}
+		if ok {
+			condition := NewHubCondition(operatorsv1.Progressing, metav1.ConditionTrue, NewComponentReason, "Created new resource")
+			SetHubCondition(&m.Status, *condition)
+		}
+	}
+	return nil
+}
+
 func updatePausedCondition(m *operatorsv1.MultiClusterHub) {
 	c := GetHubCondition(m.Status, operatorsv1.Progressing)
 
@@ -634,3 +714,12 @@ func remove(list []string, s string) []string {
 	}
 	return list
 }
+
+// mergeErrors combines errors into a single string
+func mergeErrors(errs []error) string {
+	errStrings := []string{}
+	for _, e := range errs {
+		errStrings = append(errStrings, e.Error())
+	}
+	return strings.Join(errStrings, " ; ")
+}
