@@ -1,4 +1,5 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package utils
 
@@ -42,20 +43,27 @@ var (
 	// DynamicKubeClient ...
 	DynamicKubeClient = NewKubeClientDynamic("", "", "")
 
-	// ImageOverridesCMBadImageName...
+	// ImageOverridesCMBadImageName ...
 	ImageOverridesCMBadImageName = "bad-image-ref"
 
 	// GVRCustomResourceDefinition ...
 	GVRCustomResourceDefinition = schema.GroupVersionResource{
 		Group:    "apiextensions.k8s.io",
-		Version:  "v1beta1",
+		Version:  "v1",
 		Resource: "customresourcedefinitions",
 	}
 
+	// GVRClusterManager ...
+	GVRClusterManager = schema.GroupVersionResource{
+		Group:    "operator.open-cluster-management.io",
+		Version:  "v1",
+		Resource: "clustermanagers",
+	}
+
 	// GVRObservability ...
 	GVRObservability = schema.GroupVersionResource{
 		Group:    "observability.open-cluster-management.io",
-		Version:  "v1beta1",
+		Version:  "v1beta2",
 		Resource: "multiclusterobservabilities",
 	}
 
@@ -71,6 +79,12 @@ var (
 		Version:  "v1",
 		Resource: "subscriptions",
 	}
+	// GVRHiveConfig ...
+	GVRHiveConfig = schema.GroupVersionResource{
+		Group:    "hive.openshift.io",
+		Version:  "v1",
+		Resource: "hiveconfigs",
+	}
 	// GVRSub ...
 	GVRSub = schema.GroupVersionResource{
 		Group:    "operators.coreos.com",
@@ -130,8 +144,15 @@ var (
 		Version:  "v1alpha1",
 		Resource: "baremetalassets",
 	}
+
+	// GVRDiscoveryConfig
+	GVRDiscoveryConfig = schema.GroupVersionResource{
+		Group:    "discovery.open-cluster-management.io",
+		Version:  "v1alpha1",
+		Resource: "discoveryconfigs",
+	}
 	// DefaultImageRegistry ...
-	DefaultImageRegistry = "quay.io/stolostron"
+	DefaultImageRegistry = "quay.io/open-cluster-management"
 	// DefaultImagePullSecretName ...
 	DefaultImagePullSecretName = "multiclusterhub-operator-pull-secret"
 
@@ -150,8 +171,11 @@ var (
 	// OCMSubscriptionName ...
 	OCMSubscriptionName = os.Getenv("name")
 
-	// AppSubName ClusterImageSets subscription name
-	AppSubName = "hive-clusterimagesets-subscription-fast-0"
+	// HiveConfigName ...
+	HiveConfigName = "hive"
+
+	// AppSubName console-chart-sub where clusterset pause is set
+	AppSubName = "console-chart-sub"
 
 	// SubList contains the list of subscriptions to delete
 	SubList = [...]string{
@@ -161,16 +185,19 @@ var (
 	}
 
 	// AppSubSlice ...
-	AppSubSlice = [...]string{"application-chart-sub", "cert-manager-sub",
-		"cert-manager-webhook-sub", "configmap-watcher-sub", "console-chart-sub",
+	AppSubSlice = [...]string{"application-chart-sub", "assisted-service-sub",
+		"console-chart-sub", "policyreport-sub", "discovery-operator-sub",
 		"grc-sub", "kui-web-terminal-sub", "management-ingress-sub",
-		"rcm-sub", "search-prod-sub", "topology-sub"}
+		"rcm-sub", "search-prod-sub"}
 
 	// CSVName ...
 	CSVName = "advanced-cluster-management"
 
 	// WaitInMinutesDefault ...
-	WaitInMinutesDefault = 20
+	WaitInMinutesDefault = 22
+
+	// DisableHubSelfManagementString ...
+	DisableHubSelfManagementString = "disableHubSelfManagement"
 )
 
 // GetWaitInMinutes...
@@ -360,16 +387,22 @@ func IsOwner(owner *unstructured.Unstructured, obj interface{}) bool {
 	return false
 }
 
-// CreateDefaultMCH ...
-func CreateDefaultMCH() *unstructured.Unstructured {
-	mch := NewMultiClusterHub(MCHName, MCHNamespace, "")
+// CreateMCHNotManaged ...
+func CreateMCHNotManaged() *unstructured.Unstructured {
+	mch := NewMultiClusterHub(MCHName, MCHNamespace, "", true)
 	CreateNewUnstructured(DynamicKubeClient, GVRMultiClusterHub, mch, MCHName, MCHNamespace)
 	return mch
 }
 
 // CreateMCHImageOverridesAnnotation ...
 func CreateMCHImageOverridesAnnotation(imageOverridesConfigmapName string) *unstructured.Unstructured {
-	mch := NewMultiClusterHub(MCHName, MCHNamespace, imageOverridesConfigmapName)
+	mch := NewMultiClusterHub(MCHName, MCHNamespace, imageOverridesConfigmapName, true)
+	CreateNewUnstructured(DynamicKubeClient, GVRMultiClusterHub, mch, MCHName, MCHNamespace)
+	return mch
+}
+
+func CreateDefaultMCH() *unstructured.Unstructured {
+	mch := NewMultiClusterHub(MCHName, MCHNamespace, "", false)
 	CreateNewUnstructured(DynamicKubeClient, GVRMultiClusterHub, mch, MCHName, MCHNamespace)
 	return mch
 }
@@ -411,6 +444,43 @@ func FixMCHRepo() error {
 	return KubeClient.AppsV1().Deployments(MCHNamespace).Delete(context.TODO(), MCHRepoName, metav1.DeleteOptions{})
 }
 
+// DeleteMCHRepo deletes the multiclusterhub-repo deployment
+func DeleteMCHRepo() error {
+	return KubeClient.AppsV1().Deployments(MCHNamespace).Delete(context.TODO(), MCHRepoName, metav1.DeleteOptions{})
+}
+
+func PauseMCH() error {
+	mch, err := DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
+	if err != nil {
+		return err
+	}
+	labels := mch.GetLabels()
+	if labels == nil {
+		labels = map[string]string{"mch-pause": "true"}
+	} else {
+		labels["mch-pause"] = "true"
+	}
+	mch.SetLabels(labels)
+	_, err = DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Update(context.TODO(), mch, metav1.UpdateOptions{})
+	return err
+}
+
+func UnpauseMCH() error {
+	mch, err := DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
+	if err != nil {
+		return err
+	}
+	labels := mch.GetLabels()
+	if labels == nil {
+		labels = map[string]string{"mch-pause": "false"}
+	} else {
+		labels["mch-pause"] = "false"
+	}
+	mch.SetLabels(labels)
+	_, err = DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Update(context.TODO(), mch, metav1.UpdateOptions{})
+	return err
+}
+
 // BrickKUI modifies the multiclusterhub-repo deployment so it becomes unhealthy
 func BrickKUI() (string, error) {
 	By("- Breaking kui-web-terminal")
@@ -494,6 +564,24 @@ func GetMCHStatus() (map[string]interface{}, error) {
 	return status, nil
 }
 
+// IsMCHSelfManaged returns the opposite of `spec.disableHubSelfManagement`
+func IsMCHSelfManaged() (bool, error) {
+	mch, err := DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
+	if err != nil {
+		return true, err
+	}
+	spec, ok := mch.Object["spec"].(map[string]interface{})
+	if !ok || spec == nil {
+		return true, fmt.Errorf("MultiClusterHub: %s has no 'spec' map", mch.GetName())
+	}
+	disableHubSelfManagement, ok := spec[DisableHubSelfManagementString]
+	if !ok || disableHubSelfManagement == nil {
+		return true, nil // if spec not set, default to managed
+	}
+	selfManaged := !(disableHubSelfManagement.(bool))
+	return selfManaged, nil
+}
+
 // findPhase reports whether the hub status has the desired phase and returns an error if not
 func findPhase(status map[string]interface{}, wantPhase string) error {
 	if _, ok := status["phase"]; !ok {
@@ -508,19 +596,17 @@ func findPhase(status map[string]interface{}, wantPhase string) error {
 // ValidateMCHDegraded validates the install operator responds appropriately when the install components
 // go into a degraded state after a successful install
 func ValidateMCHDegraded() error {
-	By("- Validating MultiClusterHub Degraded")
-
 	status, err := GetMCHStatus()
 	if err != nil {
 		return err
 	}
 
-	By("- Ensuring MCH is in 'pending' phase")
+	// Ensuring MCH is in 'pending' phase
 	if err := findPhase(status, "Pending"); err != nil {
 		return err
 	}
 
-	By("- Ensuring hub condition shows installation as incomplete")
+	// Ensuring hub condition shows installation as incomplete
 	if err := FindCondition(status, "Complete", "False"); err != nil {
 		return err
 	}
@@ -540,11 +626,15 @@ func ValidateDelete(clientHubDynamic dynamic.Interface) error {
 
 	appSubLink := clientHubDynamic.Resource(GVRAppSub)
 	appSubs, err := appSubLink.List(context.TODO(), listOptions)
-	Expect(err).Should(BeNil())
+	if err != nil {
+		return err
+	}
 
 	helmReleaseLink := clientHubDynamic.Resource(GVRHelmRelease)
 	helmReleases, err := helmReleaseLink.List(context.TODO(), listOptions)
-	Expect(err).Should(BeNil())
+	if err != nil {
+		return err
+	}
 
 	By("- Ensuring Application Subscriptions have terminated")
 	if len(appSubs.Items) != 0 {
@@ -560,7 +650,9 @@ func ValidateDelete(clientHubDynamic dynamic.Interface) error {
 	By("- Ensuring MCH Repo deployment has been terminated")
 	deploymentLink := clientHubDynamic.Resource(GVRDeployment).Namespace(MCHNamespace)
 	_, err = deploymentLink.Get(context.TODO(), "multiclusterhub-repo", metav1.GetOptions{})
-	Expect(err).ShouldNot(BeNil())
+	if err != nil && !errors.IsNotFound(err) {
+		return err
+	}
 
 	By("- Ensuring MCH image manifest configmap is terminated")
 	labelSelector = fmt.Sprintf("ocm-configmap-type=%s", "image-manifest")
@@ -580,13 +672,25 @@ func ValidateDelete(clientHubDynamic dynamic.Interface) error {
 
 	By("- Validating CRDs were deleted")
 	crds, err := getCRDs()
-	Expect(err).Should(BeNil())
+	if err != nil {
+		return err
+	}
 
 	for _, crd := range crds {
 		_, err = DynamicKubeClient.Resource(GVRCustomResourceDefinition).Get(context.TODO(), crd, metav1.GetOptions{})
 		Expect(err).ToNot(BeNil())
 	}
 
+	By("- Validating ClusterManager was deleted")
+	clusterManagerLink := clientHubDynamic.Resource(GVRClusterManager)
+	_, err = clusterManagerLink.Get(context.TODO(), "cluster-manager", metav1.GetOptions{})
+	Expect(err).ShouldNot(BeNil())
+
+	By("- Validating HiveConfig was deleted")
+	hiveConfigLink := clientHubDynamic.Resource(GVRHiveConfig)
+	_, err = hiveConfigLink.Get(context.TODO(), HiveConfigName, metav1.GetOptions{})
+	Expect(err).ShouldNot(BeNil())
+
 	if runCleanUpScript() {
 		By("- Running documented clean up script")
 		workingDir, err := os.Getwd()
@@ -598,7 +702,7 @@ func ValidateDelete(clientHubDynamic dynamic.Interface) error {
 		if err != nil {
 			log.Fatal(err)
 		}
-		out, err := exec.Command("/bin/sh", cleanupPath).Output() // #nosec G204 (Subprocess launched with variable)
+		out, err := exec.Command("/bin/sh", cleanupPath).Output()
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -754,14 +858,18 @@ func ValidateMCH() error {
 	}
 
 	By("- Checking Imported Hub Cluster")
-	err = ValidateManagedCluster(true)
-	Expect(err).Should(BeNil())
+	if os.Getenv("MOCK") != "true" {
+		selfManaged, err := IsMCHSelfManaged()
+		Expect(err).Should(BeNil())
+		err = ValidateManagedCluster(selfManaged)
+		Expect(err).Should(BeNil())
+	}
 
 	currentVersion, err := GetCurrentVersionFromMCH()
 	Expect(err).Should(BeNil())
 	v, err := semver.NewVersion(currentVersion)
 	Expect(err).Should(BeNil())
-	c, err := semver.NewConstraint(">= 2.2.12")
+	c, err := semver.NewConstraint(">= 2.3.4")
 	Expect(err).Should(BeNil())
 	if c.Check(v) {
 		By("- Ensuring image manifest configmap is created")
@@ -937,12 +1045,11 @@ func validateManagedClusterConditions() error {
 func ToggleDisableHubSelfManagement(disableHubSelfImport bool) error {
 	mch, err := DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
 	Expect(err).To(BeNil())
-	disableHubSelfManagementString := "disableHubSelfManagement"
-	mch.Object["spec"].(map[string]interface{})[disableHubSelfManagementString] = disableHubSelfImport
+	mch.Object["spec"].(map[string]interface{})[DisableHubSelfManagementString] = disableHubSelfImport
 	mch, err = DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Update(context.TODO(), mch, metav1.UpdateOptions{})
 	Expect(err).To(BeNil())
 	mch, err = DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
-	if disableHubSelfManagement := mch.Object["spec"].(map[string]interface{})[disableHubSelfManagementString].(bool); disableHubSelfManagement != disableHubSelfImport {
+	if disableHubSelfManagement := mch.Object["spec"].(map[string]interface{})[DisableHubSelfManagementString].(bool); disableHubSelfManagement != disableHubSelfImport {
 		return fmt.Errorf("Spec was not updated")
 	}
 	return nil
@@ -963,12 +1070,47 @@ func ToggleDisableUpdateClusterImageSets(disableUpdateCIS bool) error {
 	return nil
 }
 
+// UpdateAnnotations
+func UpdateAnnotations(annotations map[string]string) {
+	mch, err := DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Get(context.TODO(), MCHName, metav1.GetOptions{})
+	Expect(err).To(BeNil())
+	mch.SetAnnotations(annotations)
+	mch, err = DynamicKubeClient.Resource(GVRMultiClusterHub).Namespace(MCHNamespace).Update(context.TODO(), mch, metav1.UpdateOptions{})
+	Expect(err).To(BeNil())
+}
+
 // ValidateClusterImageSetsSubscriptionPause validates that the console-chart-sub created ClusterImageSets subscription is either subscription-pause true or false
 func ValidateClusterImageSetsSubscriptionPause(expected string) error {
 	appsub, err := DynamicKubeClient.Resource(GVRAppSub).Namespace(MCHNamespace).Get(context.TODO(), AppSubName, metav1.GetOptions{})
 	Expect(err).To(BeNil())
-	if subscriptionPauseValue := appsub.GetLabels()["subscription-pause"]; subscriptionPauseValue != expected {
-		return fmt.Errorf("subscription-pause label is not correct")
+	spec, ok := appsub.Object["spec"].(map[string]interface{})
+	if !ok || spec == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'spec' map", appsub.GetName())
+	}
+	packageoverrides_outer, ok := spec["packageOverrides"].([]interface{})
+	if !ok || packageoverrides_outer == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'packageoverrides' outer map", appsub.GetName())
+	}
+	packageoverrides_outer_first := packageoverrides_outer[0].(map[string]interface{})
+	packageoverrides_inner, ok := packageoverrides_outer_first["packageOverrides"].([]interface{})
+	if !ok || packageoverrides_inner == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'packageoverrides' inner map", appsub.GetName())
+	}
+	packageoverrides_inner_first := packageoverrides_inner[0].(map[string]interface{})
+	value, ok := packageoverrides_inner_first["value"].(map[string]interface{})
+	if !ok || value == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'value' map", appsub.GetName())
+	}
+	clusterimageset, ok := value["clusterImageSets"].(map[string]interface{})
+	if !ok || clusterimageset == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'clusterimageset' map", appsub.GetName())
+	}
+	subscriptionPauseValue, ok := clusterimageset["subscriptionPause"]
+	if !ok || subscriptionPauseValue == nil {
+		return fmt.Errorf("MultiClusterHub: %s has no 'subscriptionPauseValue'", appsub.GetName())
+	}
+	if subscriptionPauseValue != expected {
+		return fmt.Errorf("subscriptionPause attribute is not correct")
 	}
 	return nil
 }
@@ -1074,6 +1216,41 @@ func GetCurrentVersionFromMCH() (string, error) {
 	return version.(string), nil
 }
 
+// CreateDiscoveryConfig ...
+func CreateDiscoveryConfig() {
+	By("- Creating DiscoveryConfig CR if it does not exist")
+
+	_, err := DynamicKubeClient.Resource(GVRDiscoveryConfig).Namespace(MCHNamespace).Get(context.TODO(), "discoveryconfig", metav1.GetOptions{})
+	if err == nil {
+		return
+	}
+
+	discoveryConfigByte, err := ioutil.ReadFile("../resources/discoveryconfig.yaml")
+	Expect(err).To(BeNil())
+
+	discoveryConfig := &unstructured.Unstructured{Object: map[string]interface{}{}}
+	err = yaml.Unmarshal(discoveryConfigByte, &discoveryConfig.Object)
+	Expect(err).To(BeNil())
+
+	_, err = DynamicKubeClient.Resource(GVRDiscoveryConfig).Namespace(MCHNamespace).Create(context.TODO(), discoveryConfig, metav1.CreateOptions{})
+	Expect(err).To(BeNil())
+}
+
+// DeleteDiscoveryConfig ...
+func DeleteDiscoveryConfig() {
+	By("- Deleting DiscoveryConfig CR if it exists")
+
+	discoveryConfigByte, err := ioutil.ReadFile("../resources/discoveryconfig.yaml")
+	Expect(err).To(BeNil())
+
+	discoveryConfig := &unstructured.Unstructured{Object: map[string]interface{}{}}
+	err = yaml.Unmarshal(discoveryConfigByte, &discoveryConfig.Object)
+	Expect(err).To(BeNil())
+
+	err = DynamicKubeClient.Resource(GVRDiscoveryConfig).Namespace(MCHNamespace).Delete(context.TODO(), discoveryConfig.GetName(), metav1.DeleteOptions{})
+	Expect(err).To(BeNil())
+}
+
 // CreateObservabilityCRD ...
 func CreateObservabilityCRD() {
 	By("- Creating Observability CRD if it does not exist")
@@ -1189,6 +1366,9 @@ func getCRDs() ([]string, error) {
 	files, err := ioutil.ReadDir(crdDir)
 	Expect(err).To(BeNil())
 	for _, file := range files {
+		if filepath.Ext(file.Name()) != ".yaml" {
+			continue
+		}
 		filePath := path.Join(crdDir, file.Name())
 		src, err := ioutil.ReadFile(filepath.Clean(filePath)) // #nosec G304 (filepath cleaned)
 		if err != nil {
@@ -1210,3 +1390,15 @@ func getCRDs() ([]string, error) {
 	}
 	return crds, nil
 }
+
+// CoffeeBreak ...
+func CoffeeBreak(minutes int) {
+	log.Println(fmt.Sprintf("Starting coffee break for %d minutes...\n", minutes))
+	slept_minutes := 0
+	for slept_minutes < minutes {
+		time.Sleep(time.Duration(1) * time.Minute)
+		slept_minutes += 1
+		log.Println(fmt.Sprintf("... slept %d minutes...\n", slept_minutes))
+	}
+	log.Println(fmt.Sprintf("... ending coffee break after %d minutes!\n", slept_minutes))
+}
